@@ -225,14 +225,14 @@ class RSAPublicKey(_RSAKey, base.BasePublicKey):
 def _get_padding(pad):
     _pad = paddings[pad.__class__]
     mhash = Hash(pad.mgf.hash.name,
-                 digest_size=pad.mgf.hash.digest_size)._hasher
+                 digest_size=pad.mgf.hash.digest_size)  #._hasher
     _mgf = lambda x, y: paddings[pad.mgf.__class__](x, y, mhash)
     return _pad, _mgf
 
 
 def _get_cipher(key, pad):
     _pad, _mgf = _get_padding(pad)
-    phash = Hash(pad.hash.name, digest_size=pad.hash.digest_size)._hasher
+    phash = Hash(pad.hash.name, digest_size=pad.hash.digest_size)  #._hasher
     return _pad(key, hashAlgo=phash, mgfunc=_mgf)
 
 
@@ -285,20 +285,33 @@ def _get_signer(key, pad):
     _pad, _mgf = _get_padding(pad)
     if pad.salt_len is None:
 
-        # create a custom sign/verify wrapper to preserve consistency:
-        # pyca/cryptography follows the OpenSSL quirk where the default
-        # salt length is maximized and doesn't match with the size of the
-        # digest applied to the message.
+        @staticmethod
         def sign_or_verify(msghash, signature=None):
+            """
+            Custom sign/verify wrapper over PSS to preserve consistency:
+            pyca/cryptography follows the OpenSSL quirk where the default
+            salt length is maximized and doesn't match with the size of the
+            digest applied to the message.
+
+            Args:
+                msghash: The Hash object used to digest the message to sign.
+                signature: The signature as bytes object.
+                    If signature is None, signing is performed, otherwise
+                    verification is performed.
+
+            Returns:
+                signature as bytes object if signature argument is None.
+                None if signature argument is provided.
+            """
             salt_len = key.size_in_bytes() - msghash.digest_size - 2
             sigver = _pad(key, mgfunc=_mgf, saltLen=salt_len)
             if signature is None:
                 return sigver.sign(msghash)
             return sigver.verify(msghash, signature)
 
-        # create an object placeholder object to shadow the signer/verifier.
+        # create an object placeholder object to hold the signer/verifier.
         funcs = dict(sign=sign_or_verify, verify=sign_or_verify)
-        svobj = type('_SigVerCtx', (), funcs)
+        svobj = type('_OpenSSLStyleSigVer', (), funcs)()
         return svobj
 
     return _pad(key, mgfunc=_mgf, saltLen=pad.salt_len)
@@ -315,10 +328,7 @@ class RSASignerCtx(SigVerContext):
 
         Args:
             msghash:
-                `msghash` must be an instance of `BaseHash` and
-                must be instantiated from the same backend as that
-                of the RSA key. Refer to `Hash.new` function's
-                documentation.
+                It must be a `Hash` object, used to digest the message to sign.
 
         Returns:
             signature of the message as bytes object.
@@ -329,12 +339,8 @@ class RSASignerCtx(SigVerContext):
             ValueError: if the key is not long enough to sign the
                 massage.
         """
-        if not isinstance(msghash, Hash):
-            raise TypeError(
-                'the message hashing object must be instantiated '
-                'from the same backend as that of the RSA key.', )
         try:
-            return self._sig.sign(msghash._hasher)
+            return self._sig.sign(msghash)
         except ValueError as e:
             raise ValueError('RSA key is not long enough') from e
 
@@ -345,10 +351,7 @@ class RSAVerifierCtx(SigVerContext):
 
         Args:
             msghash:
-                `msghash` must be an instance of `BaseHash` and
-                must be instantiated from the same backend as that
-                of the RSA key. Refer to `Hash.new` function's
-                documentation.
+                It must be a `Hash` object, used to digest the message to sign.
 
             signature:
                 signature must be a `bytes` or `bytes-like` object.
@@ -357,13 +360,7 @@ class RSAVerifierCtx(SigVerContext):
             None
 
         Raises:
-            TypeError: if the `msghash` object is not from the same
-                backend.
             SignatureError: if the `signature` was incorrect.
         """
-        if not isinstance(msghash, Hash):
-            raise TypeError(
-                'the message hashing object must be instantiated '
-                'from the same backend as that of the RSA key.', )
-        if not self._sig.verify(msghash._hasher, signature):
-            raise exc.SignatureError from e
+        if not self._sig.verify(msghash, signature):
+            raise exc.SignatureError
